/*
 * io.c
 *
 *  	Created on: 25.04.2013
 *      Author: 	Zippi
 *      File:		PWM Driver
 *
 *      Beschreibung:
 *      	Inizialisiert die PWM Funktionen.
 *
 *      Beispiel (einfacher PWM fr z.B. LEDs):
 *				pwm_config(0, UPDATE_DUTY, START_HIGH, LEFT_ALIGNED, 1);	// Configuriere PWM[0], Funktion pwm_set_next updatet das Duty Register, Startet mit High, CLK/2
 *				pwm_set_max(0, 0xffff);										// Setzt den maximalen Zhlerwert auf 65535
 *				pwm_enable(0);												// Aktiviert PWM[0]
 *				pwm_set_next(0, 0x8000);									// Setzt das Duty Register auf 32768 (PWM mit 50% High, 50% Low)
 *
 */
#include <avr32/io.h>
#include "uc3_pwm.h"

void pwm_config(unsigned char uiChannel, char cPeriod, char cPolarity, char cAlignment, unsigned char ucPrescaler)
{
	AVR32_PWM.channel[uiChannel].cmr =	cPeriod << AVR32_PWM_CPD_OFFSET | 	// Channel Update Period
										cPolarity << AVR32_PWM_CPOL_OFFSET | 	// Channel Polarity
										cAlignment << AVR32_PWM_CALG_OFFSET | 	// Channel Alignment
										ucPrescaler << AVR32_PWM_CPRE_OFFSET ; 	// Channel Pre-scaler
}

void pwm_clk_pre(char cPrea, char cDiva, char cPreb, char cDivb)
{
	AVR32_PWM.mr =	cPreb << AVR32_PWM_PREB_OFFSET | 	// PREB
					cDivb << AVR32_PWM_DIVB_OFFSET |	// DIVB: CLKB Divide Factor
					cPrea << AVR32_PWM_PREA_OFFSET |	// PREA
					cDiva << AVR32_PWM_DIVA_OFFSET ;	// DIVA: CLKA Divide Factor
}

unsigned char pwm_enable(unsigned char uiChannel)
{
	AVR32_PWM.ena =	1 << uiChannel ; 	// Enable Channel X

	if((AVR32_PWM.sr & (1 << uiChannel))==0)
		return 0;						// False
	
	return 1;							// True
}

unsigned char pwm_disable(unsigned char uiChannel)
{
	AVR32_PWM.dis =	1 << uiChannel ; 	// Enable Channel X

	if((AVR32_PWM.sr & (1 << uiChannel))==0)
		return 1;					// True
	
	return 0;						// False
}

void pwm_set_max(unsigned char uiChannel, unsigned int uiMax_value)
{
	AVR32_PWM.channel[uiChannel].cprd = uiMax_value;
}

void pwm_set_duty(unsigned char uiChannel, unsigned int uiDuty_value)
{
	AVR32_PWM.channel[uiChannel].cdty = uiDuty_value;
}

void pwm_set_next(unsigned char uiChannel, unsigned int uiNext_value)
{
	AVR32_PWM.channel[uiChannel].cupd = uiNext_value;
}